@@ -2,10 +2,9 @@ package Plack::Request;
 use strict;
 use warnings;
 use 5.008_001;
-our $VERSION = '1.0026';
-$VERSION = eval $VERSION;
+our $VERSION = '1.0035';
 
-use HTTP::Headers;
+use HTTP::Headers::Fast;
 use Carp ();
 use Hash::MultiValue;
 use HTTP::Body;
@@ -34,6 +33,7 @@ sub user        { $_[0]->env->{REMOTE_USER} }
 sub request_uri { $_[0]->env->{REQUEST_URI} }
 sub path_info   { $_[0]->env->{PATH_INFO} }
 sub path        { $_[0]->env->{PATH_INFO} || '/' }
+sub query_string{ $_[0]->env->{QUERY_STRING} }
 sub script_name { $_[0]->env->{SCRIPT_NAME} }
 sub scheme      { $_[0]->env->{'psgi.url_scheme'} }
 sub secure      { $_[0]->scheme eq 'https' }
@@ -86,18 +86,11 @@ sub _parse_query {
     my @query;
     my $query_string = $self->env->{QUERY_STRING};
     if (defined $query_string) {
-        if ($query_string =~ /=/) {
-            # Handle  ?foo=bar&bar=foo type of query
-            @query =
-                map { s/\+/ /g; URI::Escape::uri_unescape($_) }
-                map { /=/ ? split(/=/, $_, 2) : ($_ => '')}
-                split(/[&;]/, $query_string);
-        } else {
-            # Handle ...?dog+bones type of query
-            @query =
-                map { (URI::Escape::uri_unescape($_), '') }
-                split(/\+/, $query_string, -1);
-        }
+        $query_string =~ s/\A[&;]+//;
+        @query =
+            map { s/\+/ /g; URI::Escape::uri_unescape($_) }
+            map { /=/ ? split(/=/, $_, 2) : ($_ => '')}
+            split(/[&;]+/, $query_string);
     }
 
     Hash::MultiValue->new(@query);
@@ -128,7 +121,7 @@ sub headers {
     my $self = shift;
     if (!defined $self->{headers}) {
         my $env = $self->env;
-        $self->{headers} = HTTP::Headers->new(
+        $self->{headers} = HTTP::Headers::Fast->new(
             map {
                 (my $field = $_) =~ s/^HTTPS?_//;
                 ( $field => $env->{$_} );
@@ -312,7 +305,7 @@ sub _parse_request_body {
 sub _make_upload {
     my($self, $upload) = @_;
     my %copy = %$upload;
-    $copy{headers} = HTTP::Headers->new(%{$upload->{headers}});
+    $copy{headers} = HTTP::Headers::Fast->new(%{$upload->{headers}});
     Plack::Request::Upload->new(%copy);
 }
 
@@ -333,7 +326,7 @@ Plack::Request - Portable HTTP request object from PSGI env hash
       my $req = Plack::Request->new($env);
 
       my $path_info = $req->path_info;
-      my $query     = $req->param('query');
+      my $query     = $req->parameters->{query};
 
       my $res = $req->new_response(200); # new Plack::Response
       $res->finalize;
@@ -418,6 +411,11 @@ Similar to C<path_info> but returns C</> in case it is empty. In other
 words, it returns the virtual path of the request URI after C<<
 $req->base >>. See L</"DISPATCHING"> for details.
 
+=item query_string
+
+Returns B<QUERY_STRING> in the environment. This is the undecoded
+query string in the request URI.
+
 =item script_name
 
 Returns B<SCRIPT_NAME> in the environment. This is the absolute path
@@ -460,7 +458,7 @@ strings that are sent by clients and are URI decoded.
 If there are multiple cookies with the same name in the request, this
 method will ignore the duplicates and return only the first value. If
 that causes issues for you, you may have to use modules like
-CGI::Simple::Cookie to parse C<$request->header('Cookies')> by
+CGI::Simple::Cookie to parse C<< $request->header('Cookies') >> by
 yourself.
 
 =item query_parameters
@@ -505,7 +503,7 @@ Returns C<REMOTE_USER> if it's set.
 
 =item headers
 
-Returns an L<HTTP::Headers> object containing the headers for the current request.
+Returns an L<HTTP::Headers::Fast> object containing the headers for the current request.
 
 =item uploads
 
@@ -541,8 +539,15 @@ Shortcut to $req->headers->user_agent.
 
 Returns GET and POST parameters with a CGI.pm-compatible param
 method. This is an alternative method for accessing parameters in
-$req->parameters. Unlike CGI.pm, it does I<not> allow
-setting or modifying query parameters.
+$req->parameters just in case you want the compatibility with
+CGI.pm objects.
+
+You are B<not recommended> to use this method since it is easy to
+misuse in a list context such as inside a hash constructor or method
+arguments. Use C<parameters> and Hash::MultiValue instead.
+
+Unlike CGI.pm, it does I<not> allow setting or modifying query
+parameters.
 
     $value  = $req->param( 'foo' );
     @values = $req->param( 'foo' );
@@ -666,11 +671,11 @@ up to your application or framework. Also, C<cookies> hash reference
 now returns I<strings> for the cookies rather than CGI::Simple::Cookie
 objects, which means you no longer have to write a wacky code such as:
 
-  $v = $req->cookie->{foo} ? $req->cookie->{foo}->value : undef;
+  $v = $req->cookies->{foo} ? $req->cookies->{foo}->value : undef;
 
 and instead, simply do:
 
-  $v = $req->cookie->{foo};
+  $v = $req->cookies->{foo};
 
 =head1 AUTHORS
 
